VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "ExcelApplication"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False

'COLLECTION CLASS MODULE TO OPEN EXCEL AND PROVIDE A COLLECTION
'OBJECT TO PROCESS A SET OF ExcelWorkbook OBJECTS.

'There should be nothing here that is specific to any
'particular workbook or project other than a Public boolean,
'canceled' in the main program.

'The Microsoft Excel Object Library, (EXCEL9.OLB, EXCEL11.OLB, etc.), must be included in the
'project reference section.

' Philip Bolt, 2005
'Last updated: 18/4/05

Option Explicit
    
'The Collection defined below is Private and used only to close down
'its members when the ExcelApplication object is terminated.
'If its declaration is made Public, it will be available in the main program.

Private xAp As Object                       'Excel application object.
Private Excel_was_not_running As Boolean    'Flag for Excel open.
Private ExcelBooks As New Collection        'Books collection.

'Declare necessary API routine to check on Excel.
Private Declare Function FindWindow Lib "user32" Alias _
"FindWindowA" (ByVal lpClassName As String, _
               ByVal lpWindowName As Long) As Long
               
Private Declare Function SendMessage Lib "user32" Alias _
"SendMessageA" (ByVal hwnd As Long, ByVal wMsg As Long, _
               ByVal wParam As Long, _
               ByVal lParam As Long) As Long

'Object constructor.
'Run when the object is created.

Private Sub Class_Initialize()

    Static set_up As Boolean    'Flag for object set-up, (only one allowed.)
    Const WM_USER = 1024
    Dim hwnd As Long
    
    If Not set_up Then
        hwnd = detect_Excel     'See if Excel is running.

        If hwnd = 0 Then                    'Excel is not running, so
            Excel_was_not_running = True    'flag this for close-down.
            Set xAp = CreateObject("Excel.Application")   'Open it.
        Else                                'Excel is running, so
            Excel_was_not_running = False   'flag this situation for close-down and
            SendMessage hwnd, WM_USER + 18, 0, 0    'enter it in the Running Object Table.
        End If

        set_up = True
    End If
    
End Sub

'Procedure dectects a running Excel and flags it.
'Function name:    detect_Excel()

Private Function detect_Excel()

    Dim hwnd As Long
    hwnd = FindWindow("XLMAIN", 0)  'If Excel is running this API call returns
                                    'its handle.
    detect_Excel = hwnd
   
End Function

'Routine to add a new workbook object to the collection.
'The ExcelWorkbook object is returned by the function and
'is used to define the ExcelWorkbook object in the main program.

'Function name: Add(ByVal nme As String)
'Parameter:     nme;   the full path book name.

Public Function Add(ByVal nme As String) As ExcelWorkbook

    Dim xBk As New ExcelWorkbook    'Dimension an 'ExcelWorkbook' object.
    
    xBk.open_book nme               'Open the workbook and,
    If Not canceled Then            'if it works,
        ExcelBooks.Add xBk          'add it to the collection,
        Set Add = xBk               'returning the book object.
    Else
        Set Add = Nothing
    End If

End Function

'Routine to delete a workbook object from the collection.

'Procedure name:    Delete(ByVal nme As String)
'Parameter:         nme;   the book name.

Public Sub Delete(ByVal nme As String)

    Dim xBk As New ExcelWorkbook    'Dimension an 'ExcelWorkbook' object and
    Dim n_bk, j As Integer          'the number of books and the loop variable.
    
    n_bk = ExcelBooks.Count     'Store the number of workbooks.
    If n_bk > 0 Then        'There are some books in the collection, so
        For j = 1 To n_bk   'check each one.
            If ExcelBooks(j).name = nme Then 'this is the one, so
                Set xBk = ExcelBooks(j)         'set the object variable to the book,
                xBk.close_book                  'close it,
                ExcelBooks.Remove j             'remove it from the collection and
                Set xBk = Nothing               'release the handle.
            End If
        Next
    End If

End Sub

'Routine to set up an enumerator for the collection.
'NewEnum must return the IUnknown interface of a
'collection's enumerator.

'This function is hidden, (Select Tools, Procedure Attributes, Select 'NewEnum'
'                          Advanced, Check 'Hide this member', set 'Procedure ID' to -4)

Public Function NewEnum() As IUnknown
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"

   Set NewEnum = ExcelBooks.[_NewEnum]
   
End Function

'Routine to close down an ExcelApplication object.
'Procedure name:    Class_Terminate()

Private Sub Class_Terminate()
 
    Dim xwb As ExcelWorkbook    'Dimension an 'ExcelWorkbook' object.
    
    For Each xwb In ExcelBooks  'Process each workbook in the collection,
        xwb.save_book           'save it and
        xwb.close_book          'close it.
    Next

    If Excel_was_not_running Then
        xAp.Quit    'Close Excel unless it was running at the start.
    End If

End Sub

